body{font-family:-apple-system,BlinkMacSystemFont,"Segoe UI","Roboto","Oxygen","Ubuntu","Cantarell","Fira Sans","Droid Sans","Helvetica Neue",sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}code{font-family:source-code-pro,Menlo,Monaco,Consolas,"Courier New",monospace}html{position:relative;min-height:100%}body{border:0;margin:0;padding-bottom:40px}.App{text-align:center}.App-logo{height:40vmin;pointer-events:none}@media (prefers-reduced-motion:no-preference){.App-logo{-webkit-animation:App-logo-spin 20s linear infinite;animation:App-logo-spin 20s linear infinite}}.App-header{background-color:#282c34;min-height:100vh;display:flex;flex-direction:column;align-items:center;justify-content:center;font-size:calc(10px + 2vmin);color:#fff}.App-link{color:#61dafb}@-webkit-keyframes App-logo-spin{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}@keyframes App-logo-spin{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}.damynavbar-right{float:right;flex-direction:row}.damynavbar-right .nav-link{padding-right:.5rem;padding-left:.5rem}span.daactivetext{color:#ff0}span.daactivetext:hover{color:#ff9}.da-pad-for-navbar{padding-top:66px}@media (max-width:767px){.danavbar-title{max-width:70%;overflow:hidden}}.da-page-header{padding-bottom:9px;margin:40px 0 20px;border-bottom:1px solid #eee}.daterm,.daterm:hover{cursor:pointer;color:#408e30}.daterm:hover{text-decoration:none}.dabackicon{background:none;cursor:pointer;border:none;padding:0;margin:0 0 0 -5px}a.dabackbuttoncolor:focus,a.dabackbuttoncolor:hover{text-decoration:none}.dabackbuttoncolor{-webkit-appearance:none}.daback{margin-left:.1em}.form-group.darequired .danolabel:before,.form-group.darequired .dawidecol:before{position:absolute;content:"*";color:red;top:0;left:.2rem}.form-group.darequired .form-label:after{content:"*";color:red;margin-left:.2em}.form-group.darequired .da-form-label:after{position:absolute;content:"*";color:red}@media (max-width:767px){.form-group.darequired .da-form-label:after{bottom:.7rem}}@media (min-width:768px){.form-group.darequired .da-form-label:after{top:5px;right:5px}}.form-group.darequired.greyedout .da-form-label:after{display:none}.form-group.dagreyedout .da-form-label{color:#999}.dawide .form-group.darequired .da-form-label:after{bottom:.7rem!important;top:auto!important;right:auto!important}.dahuge{font-size:3rem}.daiconfloat{image-orientation:from-image;float:right}.daclear{clear:both}
/*# sourceMappingURL=main.347459b5.chunk.css.map */