{%- set footer_content = get_part('change password page footer', get_part('global footer')) %}
{%- set page_title = get_part('change password page title', word('Change password')) %}
{%- set tab_title = get_part('change password page tab title', word('Change password')) %}
{%- set extra_css = get_part('change password page extra css') %}
{%- set extra_js = get_part('change password page extra javascript') %}
{%- extends 'flask_user/member_base.html' %}
{%- block content %}
{%- from "base_templates/form_macros.html" import render_field, render_submit_field %}
          <h1>{{ get_part('change password page heading', word('Change password')) }}</h1>
{{ get_part('change password page pre') }}
          <form action="" method="POST" class="form" role="form">
            {{ form.hidden_tag() }}
            {{ render_field(form.old_password) }}
            {{ render_field(form.new_password) }}
            {%- if user_manager.enable_retype_password %}
            {{ render_field(form.retype_password) }}
            {%- endif %}
            {{ get_part('change password page submit') }}<div class="form-actions">
              {{ render_submit_field(form.submit, class="btn " + config['BUTTON_STYLE'] + "primary " + config['BUTTON_CLASS']) }}
              <a class="btn {{ config['BUTTON_STYLE'] }}secondary {{ config['BUTTON_CLASS'] }}" href="{{ url_for('user.profile') }}">{{ word('Cancel') }}</a>
            </div>
          </form>{{ get_part('change password page post') }}
{%- endblock %}
