<!DOCTYPE html>
<html>
<head>
  <title>Twilio Client Quickstart</title>
  <link rel="stylesheet" href="{{ url_for('static', filename='app/webrtc.css', v=config['DA_VERSION']) }}">
</head>
<body>
  <div id="controls">
    <div id="info">
      <p class="instructions">Twilio Client</p>
      <div id="client-name"></div>
    </div>
    <div id="call-controls">
      <p class="instructions">Make a Call:</p>
      <input id="phone-number" type="text" placeholder="Enter a phone # or client name" />
      <button id="button-call">Call</button>
      <button id="button-hangup">Hangup</button>
    </div>
    <div id="log"></div>
  </div>

  <script type="text/javascript" src="//media.twiliocdn.com/sdk/js/client/v1.2/twilio.js"></script>
  <script src="//ajax.googleapis.com/ajax/libs/jquery/2.1.4/jquery.min.js"></script>
  <script src="{{ url_for('static', filename='app/webrtc.js', v=config['DA_VERSION']) }}"></script>
</body>
</html>
